/* Last edited on 2007-09-06 09:20:32 by stolfi */

double fimm_mismatch_cor
  ( int ni,                  /* Number of images being compared. */
    int nc,                  /* Number of channels to consider in input images. */
    fimm_image_eval_t *eval, /* Image evaluator. */
    r2_t rd,                 /* Half-width of the comparison window. */
    i2_t np                  /* Number of sampling points along each axis. */
  )
  {
    /* Corresponding sample values from images {0..ni-1}. */
    in nv = ni*nc;  /* Number of values per sampling point. */
    double v[nv];   /* {v[i*nc+c]} is the value of channel {c} of image {i}. */
    double w[nv];   /* {w[i*nc+c]} is the weight of channel {c} of image {i}. */

    /* Averages per image and channel: */
    double A[nv];
    for (i = 0; i < nv; i++) { A[i] = 0; }
    
    /* Correlation matrix: */
    int nv2 = nv*nv;
    double Q[nv2];
    for (ij = 0; ij < nv2; ij++) { Q[ij] = 0; }
    
    /* Enumerate sampling points: */
    int nx = np.c[0], ny = np.c[1];
    int rx, ry;
    r2_t p; /* Sampling point. */
    for (rx = 0; rx < nx; rx++)
      { p.c[0] = rd.c[0]*((2.0*rx + 1.0)/nx - 1.0);
        for (ry = -nr.c[1]; ry <= nr.c[1]; ry++)
          { p.c[1] = rd.c[1]*((2.0*ry + 1.0)/ny - 1.0);
            /* Obtain values and weights of all channels of all images at {p}: */
            for (i = 0; i < ni; i++)
              { eval(i, &p, nc, &(v[i*nc]), &(w[i*nc])); }
            /* Compute the sampling point mass {M}: */
            /* Accumulate in {A} and {Q}: */
            for (c = 0; c < nc; c++)
              { /* Compute the variance {Q} and the mass {M} for the pair {(c,p)}: */
                fimm_pc_var_mass(ni, nc, v, w, c, &Q, &M);
                /* Accumulate: */
                sum_QM += Q*M;
                sum_M += M;
              }
          }
      }
    return (sum_M == 0 ? 0.0 : sum_QM / sum_M);
  }
