#ifndef float_image_mismatch_H
#define float_image_mismatch_H

/* Last edited on 2007-09-06 09:19:32 by stolfi */ 

double fimm_mismatch_cor
  ( int n,            /* Number of images being compared. */
    fimm_eval_t *eval, /* Image evaluator. */
    r2_t rd,           /* Half-width of the comparison window. */
    i2_t np            /* Number of sampling points along each axis. */
  );
  /* Returns a mismatch measure for images {0..n-1}, based on their
    (lack of) linear correlation.
    
    For each sampling point {p}, the procedure obtains the {nv=n*nc}
    sample values, and interprets them as the coordinates of a
    particle of mass {M(p)} in {n}-dimensional real space {R^n}. The
    particle is assumed to be an axis-aligned ellipsoid with momentum
    of inertia {q(i,p)} along each axis {i}. 
    
    The procedure computes the momentum (covariance) matrix of that
    particle cloud, and then computes its eigenvalues (principal
    moments of inertia) {e[0..n-1]}, in decreasing order. The
    resulting mismatch value is {(1 - e[0]/(e[0]+e[1]+...+e[n-1]))/(n-1)}.
    
    Note that the the result is undefined (NaN) if and only if {n} is
    1, or each image is uniform. In the second case, the point cloud
    degenerates to a point, and all eigenvalues are zero. Otherwise,
    the result is minimum (0) iff the particles are pointlike and lie
    on a straight line of {R^n}; and is maximum (1) when the cloud is
    spherical, without any preferred orientation.
    
    This measure does not change when all the samples of some image
    are modified by an affine scaling {v(i,p) = r*v(i,p) + d} (and
    their noise variances are correspondingly adjusted by {q(i,p) =
    r^2*q(i,p)}). It is adequate, for example, when images are
    considered to be equal when they differ only have been
    independently adjusted for brightness and constrast. */

#endif
