#define PROG_NAME "dm_match"
#define PROG_DESC "multiscale matching of two DNA/RNA strings"
#define PROG_VERS "1.0"

/* Last edited on 2009-02-23 22:25:15 by stolfi */

#define dm_match_C_COPYRIGHT \
  "Copyright  2005 by the State University of Campinas (UNICAMP) " \
  " and the Federal Fluminense University (UFF)"

#define PROG_HELP \
  PROG_NAME " \\\n" \
  "    [ -maxLevel {MAX_LEVEL} ] \\\n" \
  "    -initFilter " wt_table_args_HELP " \\\n" \
  "    -incrFilter " wt_table_args_HELP " \\\n" \
  "    -scores " dm_score_args_HELP " \\\n" \
  "    [ -nsub {NSUB} ] \\\n" \
  "    [ -delta {DELTA} ] [ -kappa {KAPPA} ] [ -maxUnp {MAXUNP} ] \\\n" \
  "    [ -contFrac {FRAC} ] \\\n" \
  "    -maxMatches {MAX_MATCHES} \\\n" \
  "    -minBases {MIN_BASES} \\\n" \
  "    {SEQ_FILE_A} {CIRC_A} \\\n" \
  "    {SEQ_FILE_B} {CIRC_B} \\\n" \
  "    " argparser_help_info_HELP " \\\n" \
  "    {OUT_NAME}"

#define PROG_INFO \
  "NAME\n" \
  "  " PROG_NAME " - " PROG_DESC "\n" \
  "\n" \
  "SYNOPSIS\n" \
  "  " PROG_HELP "\n" \
  "\n" \
  "DESCRIPTION\n" \
  "  This program finds matching subsegments of two given" \
  " DNA/RNA sequence files, {SEQ_FILE_A} and {SEQ_FILE_B}, by" \
  " multiscale dynamic programming with smooth filtering.\n" \
  "\n" \
  "  The two input files should be in \".seq\" format" \
  " (see the INPUT FILE FORMAT section below).  The" \
  " arguments {CIRC_A} and {CIRC_B} are either \"T\" (true)" \
  " or \"F\" (false).  If {CIRC_A} is true, the" \
  " sequence in {SEQ_FILE_A} is assumed to be circular (like a bacterial" \
  " chromosome or plasmid).  Ditto for {CIRC_B} and {SEQ_FILE_B}.\n" \
  "\n" \
  "INPUT FILE FORMAT\n" \
  dm_nucleic_file_format_INFO "\n" \
  "\n" \
  "OUTPUT FILES\n" \
  "  All output files will have names starting with {OUT_NAME}.\n" \
  "\n" \
  "OPTIONS\n" \
  PROG_INFO_OPTS "\n" \
  "\n" \
  argparser_help_info_HELP_INFO "\n" \
  "SEE ALSO\n" \
  "  dm_test_filter(1)\n" \
  "\n" \
  "AUTHOR\n" \
  "  This program was created in jun/2005 by J. Stolfi, based" \
  " on the fragment reassembly programs of Helena C. G. Leito\'s" \
  " Ph.D. thesis.\n" \
  "\n" \
  "WARRANTY\n" \
  argparser_help_info_NO_WARRANTY "\n" \
  "\n" \
  "RIGHTS\n" \
  "  " dm_match_C_COPYRIGHT ".\n" \
  "\n" \
  argparser_help_info_STANDARD_RIGHTS
  
#define PROG_INFO_OPTS \
  "  -minBases {MIN_BASES} \n" \
  "    This mandatory argument specifies the minimum number of bases" \
  " in the output matches, at the original (finest) scale.\n" \
  "\n" \
  "  -maxLevel {MAX_LEVEL} \n" \
  "    This optional argument specifies the maximum number of filtering" \
  " steps to use.  The actual maximum level may be less, depending on" \
  " the {MIN_BASES} parameter.  The default is the maximum level" \
  " implied by {MIN_BASES}.\n" \
  "\n" \
  "  -maxMatches {MAX_MATCHES} \n" \
  "    This mandatory argument specifies the maximum number of matched" \
  " subsequence pairs to produce at the original (finest) scale.  This" \
  " limit is multiplied by {2^r} at each coarser scale of level {r}.\n" \
  "\n" \
  "  -initFilter " wt_table_args_HELP " \n" \
  "  -incrFilter " wt_table_args_HELP " \n" \
  "    These mandatory arguments specify the" \
  " weights of the filter to be used at the" \
  " first filtering step and at subsequent" \
  " filtering steps.  " wt_table_args_norm_sum_INFO "\n" \
  "\n" \
  "  -nsub {NSUB}\n" \
  "    Specifies the subsampling factor (number of matching" \
  " positions per sample).  The default is \"-nsub 1\".\n" \
  "\n" \
  "  -delta {DELTA}\n" \
  "    This optional argument specifies the" \
  " amount of adjustment allowed for the X and Y coordinates of" \
  " internal rungs of each pairing, when refining a candidate" \
  " match that was mapped from a coarser scale to a finer" \
  " scale.  The default is \"-delta 3\".\n" \
  "\n" \
  "  -kappa {DELTA}\n" \
  "    This optional argument specifies the" \
  " amount of X and Y extension allowed" \
  " at either end of each pairing, when mapping a candidate" \
  " match from a coarser scale to a finer scale.  The default" \
  " is \"-kappa 6\".\n" \
  "\n" \
  "  -maxUnp {MAXUNP}\n" \
  "    This optional argument specifies the" \
  " maximum unpaired datums between any two rungs of each" \
  " pairing.  The default is 6.\n" \
  "\n" \
  "  -contFrac {FRAC} \n" \
  "    This optional argument specifies a fractional" \
  " similarity level for discarding similar" \
  " candidates.  If specified and positive," \
  " the program discards (at each scale) any" \
  " candidate {A} if there is a candidate {B} that" \
  " \"mostly contains\" {A} and has better score" \
  " than {A}.  By \"mostly contains\" we mean that the" \
  " fraction of {A} pairs contained in {B} is {FRAC}" \
  " or more.  For example, if {FRAC} is 0.95, then" \
  " {B} \"mostly contains\" {A} if 95% or more of" \
  " the pairs in {A} are also in {B}.  The default" \
  " is \"-contFrac 0.99\".\n" \
  "\n" \
  "  -scores " dm_score_args_HELP "\n" \
  "    This mandatory argument specifies the" \
  " scoring criterion to be used when evaluating" \
  " and comparing candidates.  " dm_score_args_INFO

#include <dm_basic.h>
#include <dm_seq.h>
#include <dm_nucleic.h>

#include <dm_filter.h>
#include <dm_spectrum.h>
#include <dm_score.h>

#include <msm_rung.h>
#include <msm_cand.h>
#include <msm_pairing.h>
#include <msm_multi.h>
#include <msm_image.h>
#include <msm_image_tools.h>
#include <msm_ps_tools.h>

#include <affirm.h>
#include <wt_table.h>
#include <float_image.h>
#include <float_pnm_image.h>
#include <jspnm_image.h>
#include <argparser.h>

#include <math.h>
#include <assert.h>
#include <string.h>
#include <stdlib.h>

typedef struct options_t
  { char *seqNameA;      /* Name of file with sequence {A} (minus ".bas" ext). */
    bool_t circA;        /* TRUE iff sequence {A} is circular. */
    char *seqNameB;      /* Name of file with sequence {B} (minus ".gdb" ext). */
    bool_t circB;        /* TRUE iff sequence {B} is circular. */
    /* Filtering parameters: */
    int maxLevel;        /* Maximum filtering level. */
    int minBases;        /* Min number of bases in match. */
    double_vec_t w0;     /* Filter weights for first stage (level 0 to level 1). */
    double_vec_t w1;     /* Filter weights for subsequent stages. */
    /* Multiscale matching parameters: */
    int nsub;            /* Subsampling factor. */
    int maxMatches;      /* Max number o matches to report. */
    bool_t dontRefine;   /* Do not refine the candidates after mapping (debug). */
    double contFrac;     /* Containment fraction for discarding candidates. */
    /* Scoring parameters: */
    int delta;           /* Half-width of tableau around original pairing. */
    int kappa;           /* Extension of tableau beyond ends of pairing. */
    int maxUnp;          /* Maximum unpaired datums between consecutive rungs. */
    dm_score_args_t sc;  /* Candidate scoring parameters. */
    /* Output and debugging parameters: */
    char *outName;       /* Output file name prefix (minus extensions). */
  } options_t;

int main(int argc, char**argv);

options_t *msm_get_options(int argc, char**argv);
  /* Parses the command line options, packs
    them into a {options_t} record. */

msm_cand_vec_t dm_match_get_initial_cands
  ( dm_seq_t *ap,
    dm_seq_t *bp,
    int maxCands,
    double contFrac,
    int minCover,
    dm_score_args_t *sc
  );
  /* Given two sequences {ap,bp}, finds an initial set of up to
    {maxMatches} candidates, each covering at least {minCover}
    positions in both sequences. */

void dm_match_report_level
  ( dm_seq_t *ap,
    dm_seq_t *bp,
    msm_cand_vec_t *cdv,
    bool_t refined,
    char *outName,
    dm_score_args_t *sc
  );
  /* Assumes that sequences {ap} and {bp} belong to the same level {NN}:
    Outputs several diagnostic files:
      "{name}-{NN}-ed.pgm"          Seq comparison image (datum term only).
      "{name}-{NN}-seq-x.eps"       Plot of sequence 1, filtered.
      "{name}-{NN}-seq-y.eps"       Plot of sequence 2, filtered.
      "{name}-{NN}-pwr-x.eps"       Plot of power spectrum of sequence 1, filtered.
      "{name}-{NN}-pwr-y.eps"       Plot of power spectrum of sequence 2, filtered.
      "{name}-{NN}-eq.pgm"          Seq comparison image.
      "{name}-{NN}-cd-{RTG}.pgm"    Pairing trace image for cands {cdv}.
    
    The tag {RTG} is "raw" if {refined} is false, "ref" if {refined} is true. */


int main(int argc, char**argv)
  {
    options_t *o = msm_get_options(argc, argv);

    fprintf(stderr, "reading input sequences ...\n");
    dm_seq_t a = dm_seq_read_from_nucleic_file(0, "a", o->seqNameA, o->circA);
    dm_seq_t b = dm_seq_read_from_nucleic_file(1, "b", o->seqNameB, o->circB);

    fprintf(stderr, "displaying weight tables ...\n");
    char *wname0 = wt_table_make_descr(o->w0.ne, o->w0.e, "%7.5f");
    wt_table_print(stderr, "initial", o->w0.ne, o->w0.e);
    char *wname1 = wt_table_make_descr(o->w1.ne, o->w1.e, "%7.5f");
    wt_table_print(stderr, "incremental", o->w1.ne, o->w1.e);

    fprintf(stderr, "filtering to scales %d .. %d ...\n", 0, o->maxLevel);
    dm_seq_t ar[o->maxLevel + 1];
    dm_seq_t br[o->maxLevel + 1];
    dm_filter_seq_multi(&a, o->maxLevel, &(o->w0), wname0, &(o->w1), wname1, ar, o->nsub);
    dm_filter_seq_multi(&b, o->maxLevel, &(o->w0), wname0, &(o->w1), wname1, br, o->nsub);

    /* Find the maximum effective level {M}: */
    int M = 0;
    int level;
    for (level = 0; level < o->maxLevel; level++)
      { int nsa = dm_seq_num_datums(&(ar[level]));
        int nsb = dm_seq_num_datums(&(br[level]));
        fprintf(stderr, "  level %2d  num samples: a = %8d  b = %8d\n", level, nsa, nsb);
        if ((nsa >= 2) && (nsb >= 2)) { M = level; }
      }
    fprintf(stderr, "useful scales %d .. %d ...\n", 0, M);

    /* Grab coarsest seqs {aini,bini}, position counts {naini,nbini}, flags {acirc,bcirc}: */
    dm_seq_t *aini = &(ar[M]);
    dm_seq_t *bini = &(br[M]);

    /* Plot the sample distance matrix: */
    /* !!! Should show also the optimum near-diagonal pairing? !!! */
    
    auto double rung_score(msm_seq_desc_t *ad, msm_seq_desc_t *bd, msm_rung_t g);
      /* A rung scoring function. */

    double rung_score(msm_seq_desc_t *ad, msm_seq_desc_t *bd, msm_rung_t g)
      { dm_seq_t *ap = &(ar[ad->level]);
        dm_seq_t *bp = &(br[bd->level]);
        (void)msm_seq_desc_same_seq(&(ap->sd), ad, TRUE); 
        (void)msm_seq_desc_same_seq(&(bp->sd), bd, TRUE); 
        return dm_score_rung(&(o->sc), g, ap, bp);
      }
    
    fprintf(stderr, "plotting distance matrix for initial seq pair ...\n");
    msm_image_seq_seq_score_write_named
      ( &(aini->sd), &(bini->sd), &rung_score, NULL, o->outName, "-ini-ed" );

    auto double step_score(msm_seq_desc_t *ad, msm_seq_desc_t *bd, msm_rung_t g0, msm_rung_t g1);
      /* Full step scoring function. */

    double step_score(msm_seq_desc_t *ad, msm_seq_desc_t *bd, msm_rung_t g0, msm_rung_t g1)
      { dm_seq_t *ap = &(ar[ad->level]);
        dm_seq_t *bp = &(br[bd->level]);
        (void)msm_seq_desc_same_seq(&(ap->sd), ad, TRUE); 
        (void)msm_seq_desc_same_seq(&(bp->sd), bd, TRUE); 
        return dm_score_step(&(o->sc), g0, g1, ap, bp);
      }

    fprintf(stderr, "finding initial candidates ...\n");
    int S = (1 << M);
    int maxCands = o->maxMatches*S;
    int minCover = (o->minBases + S - 1)/S;
    msm_cand_vec_t cdvini = dm_match_get_initial_cands
      ( aini, bini, maxCands, o->contFrac, minCover, &(o->sc) );

    fprintf(stderr, "writing initial candidates ...\n");
    msm_cand_vec_write_named(&cdvini, o->outName, "-ini-cd");
    msm_image_cand_vec_write_named(&cdvini, &(aini->sd), &(bini->sd), o->outName, "-ini-cd" );

    /* Extract the seq descriptors {ad[0..M], bd[0..M]}: */
    msm_seq_desc_t ad[o->maxLevel + 1];
    msm_seq_desc_t bd[o->maxLevel + 1];
    for (level = 0; level <= M; level++)
      { ad[level] = ar[level].sd;
        bd[level] = br[level].sd;
        msm_seq_desc_write(stderr, "    a = ", &(ad[level]), 1, 1, 8, "\n");
        msm_seq_desc_write(stderr, "    b = ", &(bd[level]), 1, 1, 8, "\n");
      }

    /* Multiscale matching: */

    auto void report(msm_seq_desc_t *ad, msm_seq_desc_t *bd, msm_cand_vec_t *cdv, bool_t refined);
      /* Outputs data about the raw or refined candidates {cdv} between {ad} and {bd}. */

    void report(msm_seq_desc_t *ad, msm_seq_desc_t *bd, msm_cand_vec_t *cdv, bool_t refined)
      { dm_seq_t *ap = &(ar[ad->level]);
        dm_seq_t *bp = &(br[bd->level]);
        (void)msm_seq_desc_same_seq(&(ap->sd), ad, TRUE); 
        (void)msm_seq_desc_same_seq(&(bp->sd), bd, TRUE); 
        dm_match_report_level(ap, bp, cdv, refined, o->outName, &(o->sc));
      }

    fprintf(stderr, "multiscale mapping and refining ...\n");
    msm_cand_vec_t cdvfin = msm_multi_find_matches
      ( &cdvini,
        ad, bd,
        o->maxMatches, M, o->minBases,
        o->w0.ne, o->w1.ne,
        o->contFrac, (! o->dontRefine),
        o->delta, o->kappa, TRUE, TRUE, o->maxUnp,
        &step_score,
        &report
      );

    dm_seq_multi_free_datums(ar, o->maxLevel);
    dm_seq_multi_free_datums(br, o->maxLevel);

    fprintf(stderr, "plotting pair-to-pair distances ...\n");
    dm_seq_t *afin = &(ar[0]); assert(dm_seq_is_circular(aini) == dm_seq_is_circular(afin));
    dm_seq_t *bfin = &(br[0]); assert(dm_seq_is_circular(bini) == dm_seq_is_circular(bfin));
    msm_image_seq_seq_score_write_named
      ( &(afin->sd), &(bfin->sd), &rung_score, cdvfin.e[0].pr, o->outName, "-fin-ed" );

    fprintf(stderr, "writing final candidates ...\n");
    msm_cand_vec_write_named(&cdvfin, o->outName, "-fin-cd");
    msm_image_cand_vec_write_named(&cdvfin, &(afin->sd), &(bfin->sd), o->outName, "-fin-cd");

    return 0;
  }

void dm_match_report_level
  ( dm_seq_t *ap,
    dm_seq_t *bp,
    msm_cand_vec_t *cdv,
    bool_t refined,
    char *outName,
    dm_score_args_t *sc
  )
  {
    /* Filename prefix for current level: */
    char *levName = NULL;
    demand(ap->sd.level == bp->sd.level, "inconsistent levels");
    int level = ap->sd.level;
    if (outName != NULL) { asprintf(&levName, "%s-%02d", outName, level); }

    /* Plotting the filtered sequences: */
    dm_seq_postscript_plot_named(ap, 190.0, 30.0, 1.0, levName, "-seq-x");
    dm_seq_postscript_plot_named(bp, 190.0, 30.0, 1.0, levName, "-seq-y");

    /* Plotting the power spectra of the filtered sequences: */
    dm_spectrum_postscript_plot_named(ap, 190.0, 30.0, 1.0, levName, "-pwr-x");
    dm_spectrum_postscript_plot_named(bp, 190.0, 30.0, 1.0, levName, "-pwr-y");

    /* Show the sequence comparison matrix: */

    auto double rung_score(msm_seq_desc_t *ad, msm_seq_desc_t *bd, msm_rung_t g);
      /* A rung scoring function. */

    double rung_score(msm_seq_desc_t *ad, msm_seq_desc_t *bd, msm_rung_t g)
      { (void)msm_seq_desc_same_seq(&(ap->sd), ad, TRUE); 
        (void)msm_seq_desc_same_seq(&(bp->sd), bd, TRUE); 
        return dm_score_rung(sc, g, ap, bp);
      }
    
    if (levName != NULL)
      { /* Plot pair-to-pair distances: */;
        msm_image_seq_seq_score_write_named(&(ap->sd), &(bp->sd), rung_score, NULL, levName, "-ed-cur");
      }

    int ic;

    /* Print the candidates: */
    char *refTag = (refined ? "ref" : "map");
    fprintf(stderr, "- - - - - - - - - - - - - - - - - - - - - - - - - - - - - -\n");
    fprintf(stderr, "%d %s candidates at the start of level %d:\n", cdv->ne, refTag, level);
    for (ic = 0; ic < cdv->ne; ic++)
      { fprintf(stderr, "  map[%03d]: ", ic);
        msm_cand_write(stderr, NULL, &(cdv->e[ic]), 6, 8, 6, NULL);
        fprintf(stderr, "\n");
      }
    fprintf(stderr, "- - - - - - - - - - - - - - - - - - - - - - - - - - - - - -\n");

    if (levName != NULL)
      { /* Write and plot the raw candidates: */;
        char *cdvTag = NULL;
        asprintf(&cdvTag, "-cd-%s", refTag);
        msm_cand_vec_write_named(cdv, levName, cdvTag);
        msm_image_cand_vec_write_named(cdv, &(ap->sd), &(bp->sd), levName, cdvTag);
        free(cdvTag);
      }

    if (levName != NULL) { free(levName); }
  }

msm_cand_vec_t dm_match_get_initial_cands
  ( dm_seq_t *ap,
    dm_seq_t *bp,
    int maxCands,
    double contFrac,
    int minCover,
    dm_score_args_t *sc
  )
  { double minScore = 0.0;
    
    auto double step_score(msm_seq_desc_t *ad, msm_seq_desc_t *bd, msm_rung_t g0, msm_rung_t g1);
      /* Full step scoring function. */

    double step_score(msm_seq_desc_t *ad, msm_seq_desc_t *bd, msm_rung_t g0, msm_rung_t g1)
      { (void)msm_seq_desc_same_seq(&(ap->sd), ad, TRUE); 
        (void)msm_seq_desc_same_seq(&(bp->sd), bd, TRUE); 
        return dm_score_step(sc, g0, g1, ap, bp);
      }
    
    msm_cand_vec_t cdv = msm_cand_vec_get_best_perfect
      ( &(ap->sd), &(bp->sd), step_score,
        minCover, minScore, maxCands, contFrac
      );

    fprintf(stderr, "found %d candidates at coarsest level\n", cdv.ne);
    return cdv;
  }

#define MAX_NSUB (64)
  /* To avoid humongous tableaus. */

options_t *msm_get_options(int argc, char**argv)
  {
    options_t *o = (options_t *)notnull(malloc(sizeof(options_t)), "no mem");

    argparser_t *pp = argparser_new(stderr, argc, argv);
    argparser_set_help(pp, PROG_HELP);
    argparser_set_info(pp, PROG_INFO);
    argparser_process_help_info_options(pp);

    /* Note that level 40 requires seqs with at least {2^40} bases: */
    if (argparser_keyword_present(pp, "-maxLevel"))
      { o->maxLevel = argparser_get_next_int(pp, 0, 40); }
    else
      { o->maxLevel = 40; }

    argparser_get_keyword(pp, "-minBases");
    o->minBases = argparser_get_next_int(pp, 0, INT_MAX);

     if (argparser_keyword_present(pp, "-nsub"))
      { o->nsub = argparser_get_next_int(pp, 1, MAX_NSUB); }
    else
      { o->nsub = 1; }
    
    argparser_get_keyword(pp, "-scores");
    o->sc = dm_score_args_parse(pp);

    if (argparser_keyword_present(pp, "-delta"))
      { o->delta = argparser_get_next_int(pp, 0, INT_MAX); }
    else
      { o->delta = 3; }

    if (argparser_keyword_present(pp, "-kappa"))
      { o->kappa = argparser_get_next_int(pp, 0, INT_MAX); }
    else
      { o->kappa = 6; }

    if (argparser_keyword_present(pp, "-maxUnp"))
      { o->maxUnp = argparser_get_next_int(pp, 0, INT_MAX); }
    else
      { o->maxUnp = 6; }

    argparser_get_keyword(pp, "-maxMatches");
    o->maxMatches = argparser_get_next_int(pp, 0, INT_MAX);

    argparser_get_keyword(pp, "-initFilter");
    o->w0 = wt_table_args_parse(pp, TRUE);

    argparser_get_keyword(pp, "-incrFilter");
    o->w1 = wt_table_args_parse(pp, TRUE);

    if (argparser_keyword_present(pp, "-contFrac"))
      { o->contFrac = argparser_get_next_double(pp, -1.0, +1.0); }
    else
      { o->contFrac = 0.99; }

    argparser_skip_parsed(pp);

    o->seqNameA = argparser_get_next(pp);
    o->circA = argparser_get_next_bool(pp);

    o->seqNameB = argparser_get_next(pp);
    o->circB = argparser_get_next_bool(pp);

    o->outName = argparser_get_next(pp);

    o->dontRefine = FALSE; /* DEBUG */

    argparser_finish(pp);

    return o;
  }
