/* See {dm_score.h} */
/* Last edited on 2008-04-19 12:12:14 by stolfi */

#define dm_score_C_COPYRIGHT \
  "Copyright  2006  by the State University of Campinas (UNICAMP)" \
  " and the Federal Fluminense University (UFF)"

#include <dm_basic.h>
#include <dm_seq.h>
#include <dm_score.h>

#include <msm_rung.h>

#include <argparser.h>
#include <jsmath.h>

#include <assert.h>
#include <stdio.h>
#include <math.h>

double dm_score_rung_from_datums(double eql, double dif, dm_datum_t *dx, dm_datum_t *dy)
  { double diff = dm_datum_diffsq(dx, dy);
    double ffid = 1 - diff;
    return ffid*eql + diff*dif;
  }

double dm_score_step_from_datums
  ( double eql, 
    double dif,
    dm_datum_t *d0x, 
    dm_datum_t *d0y, 
    dm_datum_t *d1x,
    dm_datum_t *d1y
  )
  { /* Compute the part {sc_diff} of score that is due to datum diffs along step: */
    double S0 = dm_score_rung_from_datums(eql,dif,d0x,d0y);
    double S1 = dm_score_rung_from_datums(eql,dif,d1x,d1y);
    return 0.5*(S0 + S1);
  }

double dm_score_step_from_increments(double brk, double skp, double unp, int kx, int ky)
  { demand(kx > 0, "X increment must be positive");
    demand(ky > 0, "Y increment must be positive");
    if ((kx == 1) && (ky == 1))
      { /* Perfect step: */ return 0.0; }
    else 
      { int kSkp = imin(kx - 1, ky - 1); 
        int kUnp = abs(kx - ky);
        return brk + kSkp*skp + kUnp*unp; 
      }
  }

double dm_score_rung(dm_score_args_t *sc, msm_rung_t g, dm_seq_t *xp, dm_seq_t *yp)
  { if (msm_rung_is_none(g))
      { return 0; }
    else
      { int x = g.c[0];
        int y = g.c[1];
        dm_datum_t dx = dm_seq_eval(xp, x);
        dm_datum_t dy = dm_seq_eval(yp, y);
        return dm_score_rung_from_datums(sc->eql, sc->dif, &dx, &dy);
      }
  }

double dm_score_step(dm_score_args_t *sc, msm_rung_t g0, msm_rung_t g1, dm_seq_t *xp, dm_seq_t *yp)
  { 
    /* Compute the datum similarity term {Sd}: */
    double Sd = dm_score_step_data_term(sc, g0, g1, xp, yp);
    /* Compute the motion penalty {Sm}: */
    double Sm = dm_score_step_move_term(sc, g0, g1);
    return Sd + Sm;
  }

double dm_score_step_data_term
  ( dm_score_args_t *sc,
    msm_rung_t g0, 
    msm_rung_t g1, 
    dm_seq_t *xp, 
    dm_seq_t *yp
  )
  { 
    /* Check whether the rungs are defined: */
    double S0 = dm_score_rung(sc, g0, xp, yp);
    double S1 = dm_score_rung(sc, g1, xp, yp);
    return 0.5*(S0 + S1);
  }

double dm_score_step_move_term(dm_score_args_t *sc, msm_rung_t g, msm_rung_t h)
  { 
    /* Check whether the ends are defined: */
    bool_t gnone = msm_rung_is_none(g);
    bool_t hnone = msm_rung_is_none(h);

    if (gnone || hnone)
      { return 0; }
    else
      { int kx = h.c[0] - g.c[0];
        int ky = h.c[1] - g.c[1];
        return dm_score_step_from_increments(sc->brk, sc->skp, sc->unp, kx, ky);
      }
  }

dm_score_args_t dm_score_args_parse(argparser_t *pp)
  { dm_score_args_t sc;
    argparser_keyword_present_next(pp, "data");
    sc.eql = argparser_get_next_double(pp,    0.0, +100.0); 
    sc.dif = argparser_get_next_double(pp, -100.0,    0.0); 
    argparser_keyword_present_next(pp, "move");
    sc.brk = argparser_get_next_double(pp, -100.0, +100.0); 
    sc.skp = argparser_get_next_double(pp, -100.0, +100.0); 
    sc.unp = argparser_get_next_double(pp, -100.0, +100.0); 
    return sc;
  }

