#ifndef dm_test_tools_H
#define dm_test_tools_H

/* Miscellaneous tools for test programs. */
/* Last edited on 2008-04-19 23:12:07 by stolfi */

#define dm_test_tools_H_COPYRIGHT \
  "Copyright  2006  by the State University of Campinas (UNICAMP)"

#include <vec.h>

#include <msm_basic.h>
#include <msm_pairing.h>
#include <msm_cand.h>
#include <msm_dyn.h>

#include <dm_seq.h>
#include <dm_score.h>

#define dm_test_tools_FONT_SIZE (10.0)

void dm_test_tools_seq_write_and_plot_named
  ( dm_seq_t *seq, 
    char *title,
    char *name,
    char *tag
  );
  /* Writes a test sequence to a disk file called "{name}{tag}.txt".
    Also writes a plot of its sample values to a disk file called
    "{name}{tag}.eps". Assumes that the sequence samples are in the
    vector {*smp}, and that the sequence's descriptor is {*seq}, with
    subsampling factor {den}. */

void dm_test_tools_seq_multi_write_and_plot_named
  ( dm_seq_t seq[],
    int maxLevel,
    char *title,
    char *name,
    char *tag
  );
  /* Assumes that {seq[i]}, for {i} in {0..maxLevel-1}, is a versions
    of sequence {seq[0]} filtered and downsampled {i} times. Calls
    {dm_test_tools_seq_write_and_plot_named(seq[i],titlei,name,tagi)}
    on each sequence {seq[i]}; where {titlei} is {title} with " level
    {i}" appended, and {tagi} is {tag} with "-{II}" appended, where
    {II} is the two-digit level {i}. */

void dm_test_tools_make_seq_pair
  ( char *borg, 
    double mutProb, 
    double delProb, 
    bool_t circ,
    dm_seq_id_t xid, 
    char *xtag,
    dm_seq_t *xP,
    dm_seq_id_t yid, 
    char *ytag,
    dm_seq_t *yP,
    msm_pairing_t **prP,
    char *outName
  );
  /* Generates two test sequences {*xP,*yP} derived from 
    the nuceotide string {borg} by mutations, insertions and deletions.
  
    The sequences will have numeric identifier
    {xid,yid}, name {xtag,ytag}, and appropriate {cmt} strings. The sequences 
    are circular iff {circ} is TRUE.  
    
    The procedure also returns in {*prP} the list of `true' rungs between
    between the two sequences, i.e. the list of pairs {ix,iy} such that
    {xP[ix]} and {yP[iy]} were both copied from the same letter of 
    {borg}, without mutation.  Beware that {**prP} will be 1-increasing but
    not 1-atomic in general.
    
    The procedure also writes the sequences to files
    "{outName}-{xtag}.bas" and "{outName}-{ytag}.bas".
    
    Each output sequence is generated idependently of the other.
    The precise meaning of the parameters {mutProb} and {delProb} is
    defined by the procedure {dm_nucleic_string_mutate}. */

void dm_test_tools_write_generated_sequence(char *b, dm_seq_t *s, char *outName);
  /* Writes the nucleotide string {b} and its numerically encoded 
    version {s} to disk files named "{outName}-{s.name}.bas" 
    and "{outName}-{s.name}.egs", respectively. */

msm_cand_t dm_test_tools_get_optimum_pairing
  ( dm_score_args_t *sc, 
    dm_seq_t *xp, 
    dm_seq_t *yp, 
    int delta, 
    msm_dyn_tableau_t *tb,
    int maxIter
  );
  /* Finds the optimum pairing between sequences {*sp} and {*yp}
    by restricted dynamic programming.  The search is restricted
    to a band that deviates at most {delta} sequence positions
    from the atomic Bresenham pairing. Uses the tableau {tb}
    (which must be pre-allocated by the user) as temporary work
    area. */

#endif

